﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;

namespace cleverbridge.Games.TicTacToe
{
    internal class Game
    {
        private readonly Player?[,] _grid;
        private readonly GameLog _log;

        public Game()
        {
            _grid = new Player?[3, 3];
            _log = new GameLog();
        }

        public GameLog Log
        {
            get
            {
                return _log;
            }
        }

        public Int32 Moves
        {
            get
            {
                var moves = 0;
                for (var row = 0; row < 3; row++)
                {
                    for (var column = 0; column < 3; column++)
                    {
                        if (_grid[row, column].HasValue)
                            moves++;
                    }
                }
                return moves;
            }
        }

        public Player? NextPlayer
        {
            get
            {
                if (Winner != null || Moves >= 9)
                    return null;

                if (Moves % 2 == 0)
                    return Player.PlayerX;
                else
                    return Player.PlayerO;
            }
        }

        public Player? this[
            Position position]
        {
            get
            {
                return this[position.Row, position.Column];
            }
        }

        public Player? this[
            Int32 row,
            Int32 column]
        {
            get
            {
                return _grid[row, column];
            }
        }

        public Player? Winner
        {
            get
            {
                foreach (var xxx in _winCombinations)
                {
                    var player = this[xxx[0]];

                    for (var index = 1; index < xxx.Count; index++)
                    {
                        if (player != this[xxx[index]])
                        {
                            player = null;
                            break;
                        }
                    }

                    if (player != null)
                        return player;
                }

                return null;
            }
        }

        public void DoBestMove()
        {
            var class3 = GetWin(NextPlayer.Value);
            if (class3 != null)
            {
                DoMove(class3);
                return;
            }

            class3 = GetWin(GetOtherPlayer(NextPlayer.Value));
            if (class3 != null)
            {
                DoMove(class3);
                return;
            }

            class3 = GetFork(NextPlayer.Value);
            if (class3 != null)
            {
                DoMove(class3);
                return;
            }

            class3 = GetFork(GetOtherPlayer(NextPlayer.Value));
            if (class3 != null)
            {
                DoMove(class3);
                return;
            }

            if (_grid[1, 1] == null)
            {
                DoMove(1, 1);
                return;
            }

            if (_grid[0, 0] == GetOtherPlayer(NextPlayer.Value) && _grid[2, 2] == null)
            {
                DoMove(2, 2);
                return;
            }

            if (_grid[0, 2] == GetOtherPlayer(NextPlayer.Value) && _grid[2, 0] == null)
            {
                DoMove(2, 0);
                return;
            }

            if (_grid[2, 0] == GetOtherPlayer(NextPlayer.Value) && _grid[0, 2] == null)
            {
                DoMove(0, 2);
                return;
            }

            if (_grid[2, 2] == GetOtherPlayer(NextPlayer.Value) && _grid[0, 0] == null)
            {
                DoMove(0, 0);
                return;
            }

            if (_grid[0, 0] == null)
            {
                DoMove(0, 0);
                return;
            }

            if (_grid[0, 2] == null)
            {
                DoMove(0, 2);
                return;
            }

            if (_grid[2, 0] == null)
            {
                DoMove(2, 0);
                return;
            }

            if (_grid[2, 2] == null)
            {
                DoMove(2, 2);
                return;
            }

            DoRandomMove();
        }

        public void DoMove(
            Position position)
        {
            DoMove(position.Row, position.Column);
        }

        public void DoMove(
            Int32 row,
            Int32 column)
        {
            if (NextPlayer != null)
            {
                _grid[row, column] = NextPlayer;

                if (NextPlayer == null)
                {
                    Log.AddEntry(Winner, Moves);
                }
            }
        }

        public void DoRandomMove()
        {
            var list = new List<Position>();

            for (var row = 0; row < 3; row++)
            {
                for (var column = 0; column < 3; column++)
                {
                    if (!_grid[row, column].HasValue)
                    {
                        list.Add(new Position(row, column));
                    }
                }
            }

            if (list.Count > 0)
            {
                var random = new Random();

                var position = list[random.Next(list.Count)];

                DoMove(position.Row, position.Column);
            }
        }

        public bool IsWinPosition(
            Int32 row,
            Int32 column)
        {
            foreach (var xxx in _winCombinations)
            {
                var player = this[xxx[0]];

                for (var index = 1; index < xxx.Count; index++)
                {
                    if (player != this[xxx[index]])
                    {
                        player = null;
                        break;
                    }
                }

                if (player != null)
                {
                    for (var index = 0; index < xxx.Count; index++)
                    {
                        if (xxx[index].Row == row && xxx[index].Column == column)
                            return true;
                    }

                    return false;
                }
            }

            return false;
        }

        public void StartNewGame()
        {
            for (var row = 0; row < 3; row++)
            {
                for (var column = 0; column < 3; column++)
                {
                    _grid[row, column] = null;
                }
            }
        }

        private Position GetFork(
            Player player)
        {
            for (var row = 0; row < 3; row++)
            {
                for (var column = 0; column < 3; column++)
                {
                    if (_grid[row, column] == null)
                    {
                        var opportunities = 0;

                        foreach (var xxx in _winCombinations)
                        {
                            var playerCount = 0;
                            var otherCount = 0;

                            foreach (var yyy in xxx)
                            {
                                if (this[yyy] == player || (yyy.Row == row && yyy.Column == column))
                                    playerCount++;
                                else if (this[yyy] != null)
                                    otherCount++;
                            }

                            if (playerCount > 1 && otherCount == 0)
                            {
                                opportunities++;
                            }
                        }

                        if (opportunities > 1)
                        {
                            return new Position(row, column);
                        }
                    }
                }
            }

            return null;
        }

        private Position GetWin(
            Player player)
        {
            foreach (var winCombination in _winCombinations)
            {
                Position empty = null;
                var playerCount = 0;

                foreach (var position in winCombination)
                {
                    if (this[position] == null)
                        empty = position;
                    else if (this[position] == player)
                        playerCount++;
                }

                if (empty != null && playerCount == 2)
                    return empty;
            }

            return null;
        }

        private static readonly ReadOnlyCollection<ReadOnlyCollection<Position>> _winCombinations;

        static Game()
        {
            var winCombinationList = new List<ReadOnlyCollection<Position>>();

            List<Position> winCombination;

            for (var row = 0; row < 3; row++)
            {
                winCombination = new List<Position>();

                for (var column = 0; column < 3; column++)
                {
                    winCombination.Add(new Position(row, column));
                }

                winCombinationList.Add(winCombination.AsReadOnly());
            }

            for (var column = 0; column < 3; column++)
            {
                winCombination = new List<Position>();

                for (var row = 0; row < 3; row++)
                {
                    winCombination.Add(new Position(row, column));
                }

                winCombinationList.Add(winCombination.AsReadOnly());
            }

            winCombination = new List<Position>();

            winCombination.Add(new Position(0, 0));
            winCombination.Add(new Position(1, 1));
            winCombination.Add(new Position(2, 2));

            winCombinationList.Add(winCombination.AsReadOnly());

            winCombination = new List<Position>();

            winCombination.Add(new Position(2, 0));
            winCombination.Add(new Position(1, 1));
            winCombination.Add(new Position(0, 2));

            winCombinationList.Add(winCombination.AsReadOnly());

            _winCombinations = winCombinationList.AsReadOnly();
        }

        private static Player GetOtherPlayer(
            Player player)
        {
            if (player == Player.PlayerX)
                return Player.PlayerO;
            else
                return Player.PlayerX;
        }
    }
}