﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace cleverbridge.Games.TicTacToe
{
    /// <summary>
    /// Interaction logic for GameControl.xaml
    /// </summary>
    public partial class GameControl : UserControl
    {
        private readonly Game _game;

        public GameControl()
        {
            InitializeComponent();

            for (var row = 0; row < 3; row++)
            {
                var rowDefinition = new RowDefinition();

                _grid.RowDefinitions.Add(rowDefinition);

                for (var column = 0; column < 3; column++)
                {
                    if (row == 0)
                    {
                        var columnDefinition = new ColumnDefinition();

                        _grid.ColumnDefinitions.Add(columnDefinition);
                    }

                    var label = new Label();

                    label.HorizontalContentAlignment = HorizontalAlignment.Center;
                    label.VerticalContentAlignment = VerticalAlignment.Center;

                    Grid.SetColumn(label, column);
                    Grid.SetRow(label, row);

                    label.MouseDown += new MouseButtonEventHandler(label_MouseDown);
                    label.MouseEnter += new MouseEventHandler(label_MouseEnter);
                    label.MouseLeave += new MouseEventHandler(label_MouseLeave);

                    _grid.Children.Add(label);
                }
            }

            _game = new Game();

            Update();
        }

        void label_MouseDown(
            object sender,
            MouseButtonEventArgs e)
        {
            if (_game.Winner != null)
            {
                return;
            }

            var label = (Label)sender;

            _game.DoMove(Grid.GetRow(label), Grid.GetColumn(label));

            Play();

            Update();
        }

        void label_MouseEnter(
            object sender,
            MouseEventArgs e)
        {
            var label = (Label)sender;

            if (label.Content == null)
            {
                if (_game.NextPlayer == Player.PlayerX)
                    label.Content = FindResource("PreviewPlayerXTile");
                else if (_game.NextPlayer == Player.PlayerO)
                    label.Content = FindResource("PreviewPlayerOTile");
            }
        }

        void label_MouseLeave(
            object sender,
            MouseEventArgs e)
        {
            var label = (Label)sender;

            if (_game[Grid.GetRow(label), Grid.GetColumn(label)] == null)
                label.Content = null;
        }

        private void Play()
        {
            while (true)
            {
                if (_game.NextPlayer == Player.PlayerX)
                {
                    if (_randomComputerPlayerXRadioButton.IsChecked.Value)
                        _game.DoRandomMove();
                    else if (_perfectComputerPlayerXRadioButton.IsChecked.Value)
                        _game.DoBestMove();
                    else
                        return;
                }
                else if (_game.NextPlayer == Player.PlayerO)
                {
                    if (_randomComputerPlayerORadioButton.IsChecked.Value)
                        _game.DoRandomMove();
                    else if (_perfectComputerPlayerORadioButton.IsChecked.Value)
                        _game.DoBestMove();
                    else
                        return;
                }
                else
                {
                    return;
                }
            }
        }

        private void _startNewGameButton_Click(
            object sender,
            RoutedEventArgs e)
        {
            _game.StartNewGame();

            Play();

            Update();
        }

        private void Update()
        {
            foreach (Label label in _grid.Children)
            {
                var player = _game[Grid.GetRow(label), Grid.GetColumn(label)];

                if (player.HasValue)
                {
                    if (player.Value == Player.PlayerX)
                    {
                        if (_game.IsWinPosition(Grid.GetRow(label), Grid.GetColumn(label)))
                            label.Content = FindResource("WonPlayerXTile");
                        else
                            label.Content = FindResource("DefaultPlayerXTile");
                    }
                    else
                    {
                        if (_game.IsWinPosition(Grid.GetRow(label), Grid.GetColumn(label)))
                            label.Content = FindResource("WonPlayerOTile");
                        else
                            label.Content = FindResource("DefaultPlayerOTile");
                    }
                }
                else
                {
                    label.Content = null;
                }
            }

            _movesRightLabel.Content = _game.Moves;

            if (_game.NextPlayer == null)
            {
                if (_game.Winner == Player.PlayerX)
                    _winnerRightLabel.Content = "Player X";
                else if (_game.Winner == Player.PlayerO)
                    _winnerRightLabel.Content = "Player O";
                else
                    _winnerRightLabel.Content = "Draw";
            }
            else
            {
                _winnerRightLabel.Content = null;
            }

            _drawLabel.Content = _game.Log.GetCount(null, null);

            _wonByPlayerOAfter6MovesLabel.Content = _game.Log.GetCount(Player.PlayerO, 6);
            _wonByPlayerOAfter8MovesLabel.Content = _game.Log.GetCount(Player.PlayerO, 8);
            _wonByPlayerOLabel.Content = _game.Log.GetCount(Player.PlayerO, null);

            _wonByPlayerXAfter5MovesLabel.Content = _game.Log.GetCount(Player.PlayerX, 5);
            _wonByPlayerXAfter7MovesLabel.Content = _game.Log.GetCount(Player.PlayerX, 7);
            _wonByPlayerXAfter9MovesLabel.Content = _game.Log.GetCount(Player.PlayerX, 9);
            _wonByPlayerXLabel.Content = _game.Log.GetCount(Player.PlayerX, null);
        }
    }
}